"""
Main entry point.
"""

import argparse

from static_npm.node import Node
from static_npm.npm import Npm
from static_npm.npx import Npx


def parse_args() -> tuple[argparse.Namespace, list[str]]:
    """Parse command-line arguments."""
    parser = argparse.ArgumentParser(description="npm static binary")
    return parser.parse_known_args()


def run(prog: Npm | Npx | Node) -> int:
    """Main entry point for the template_python_cmd package."""
    _, unknown = parse_args()
    proc = prog.run(unknown)
    rtn = proc.wait()
    return rtn



"""
Example:

live_server = NpmTool("live-server")
live_server.install()
proc = live_server.run([f"--port={PORT}", "--no-browser"])

timeout = time.time() + 120
while time.time() < timeout:
    try:
        response = httpx.get(f"http://localhost:{PORT}")
        if response.status_code == 200:
            break
    except httpx.ConnectError:
        pass
    time.sleep(1)
else:
    self.fail("Server did not start.")

proc.terminate()
"""

from pathlib import Path

from static_npm.npm import Npm
from static_npm.npx import Npx
from static_npm.paths import CACHE_DIR
from static_npm.running_process import RunningProcess


def _get_tool_dir(tool: str) -> Path:
    return CACHE_DIR / tool


class NpmTool:
    """Installs a single use npm tool and allows you to run it."""

    def __init__(self, tool: str) -> None:
        self.tool = tool
        self.tool_dir = _get_tool_dir(tool)
        self.install_attempt = False

    def install(self) -> None:
        npm = Npm()
        self.tool_dir.mkdir(exist_ok=True, parents=True)
        npm.run(["install", self.tool, "--prefix", str(self.tool_dir)])
        self.install_attempt = True

    def run(
        self, args: list[str], cwd: Path | None = None, echo=False
    ) -> RunningProcess:
        if not self.install_attempt:
            self.install()
        npx = Npx()
        proc = npx.run(
            ["--prefix", str(self.tool_dir), self.tool, *args], cwd=cwd, echo=echo
        )
        return proc



import subprocess
import threading
from pathlib import Path


class RunningProcess:
    """
    A class to manage and stream output from a running subprocess.

    This class provides functionality to execute shell commands, stream their output
    in real-time, and control the subprocess execution.
    """

    def __init__(
        self,
        command: str | list[str],
        cwd: Path | None = None,
        check: bool = False,
        auto_run: bool = True,
        echo: bool = True,
    ):
        """
        Initialize the RunningProcess instance. Note that stderr is merged into stdout!!

        Args:
            command (str): The command to execute.
            cwd (Path | None): The working directory to execute the command in.
            check (bool): If True, raise an exception if the command returns a non-zero exit code.
            auto_run (bool): If True, automatically run the command when the instance is created.
            echo (bool): If True, print the output of the command to the console in real-time.
        """
        if isinstance(command, list):
            command = subprocess.list2cmdline(command)
        self.command = command
        self.cwd = str(cwd) if cwd is not None else None
        self.buffer: list[str] = []
        self.proc: subprocess.Popen | None = None
        self.check = check
        self.auto_run = auto_run
        self.echo = echo
        self.reader_thread: threading.Thread | None = None
        self.shutdown: threading.Event = threading.Event()
        if auto_run:
            self.run()

    def run(self) -> None:
        """
        Execute the command and stream its output in real-time.

        Returns:
            str: The full output of the command.

        Raises:
            subprocess.CalledProcessError: If the command returns a non-zero exit code.
        """

        self.proc = subprocess.Popen(
            self.command,
            shell=True,
            cwd=self.cwd,
            bufsize=256,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,  # Merge stderr into stdout
            text=True,  # Automatically decode bytes to str
        )

        def output_reader():
            try:
                assert self.proc.stdout is not None
                for line in iter(self.proc.stdout.readline, ""):
                    if self.shutdown.is_set():
                        break
                    line = line.rstrip()
                    if self.echo:
                        print(line)  # Print to console in real time
                    self.buffer.append(line)
            finally:
                if self.proc.stdout:
                    self.proc.stdout.close()

        # Start output reader thread
        self.reader_thread = threading.Thread(target=output_reader, daemon=True)
        self.reader_thread.start()

    def wait(self) -> int:
        """
        Wait for the process to complete.

        Raises:
            ValueError: If the process hasn't been started.
        """
        if self.proc is None:
            raise ValueError("Process is not running.")
        rtn = self.proc.wait()
        assert self.reader_thread is not None
        self.reader_thread.join(timeout=1)
        return rtn

    def kill(self) -> None:
        """
        Immediately terminate the process with SIGKILL.

        Raises:
            ValueError: If the process hasn't been started.
        """
        if self.proc is None:
            raise ValueError("Process is not running.")
        self.shutdown.set()
        self.proc.kill()

    def terminate(self) -> None:
        """
        Gracefully terminate the process with SIGTERM.

        Raises:
            ValueError: If the process hasn't been started.
        """
        if self.proc is None:
            raise ValueError("Process is not running.")
        self.shutdown.set()
        self.proc.terminate()

    @property
    def returncode(self) -> int | None:
        if self.proc is None:
            return None
        return self.proc.returncode

    @property
    def stdout(self) -> str:
        """
        Get the complete stdout output of the process.

        Returns:
            str: The complete stdout output as a string, or None if process hasn't completed.
        """
        self.wait()
        return "\n".join(self.buffer)



"""
Unit test file.
"""

import time
import unittest

import httpx  # type: ignore

from static_npm import NpmTool

PORT = 8375


class LiveServerTester(unittest.TestCase):
    """Main tester class."""

    def test_live_server_instantiates_and_responds(self) -> None:
        """Test command line interface (CLI)."""
        live_server = NpmTool("live-server")
        live_server.install()
        proc = live_server.run([f"--port={PORT}", "--no-browser"])

        timeout = time.time() + 120
        while time.time() < timeout:
            try:
                response = httpx.get(f"http://localhost:{PORT}")
                if response.status_code == 200:
                    break
            except httpx.ConnectError:
                pass
            time.sleep(1)
        else:
            self.fail("Server did not start.")

        proc.terminate()


if __name__ == "__main__":
    unittest.main()



"""
Main entry point.
"""

import argparse

from static_npm.node import Node
from static_npm.npm import Npm
from static_npm.npm_tool import NpmTool
from static_npm.npx import Npx
from static_npm.prog_exec import run


def parse_args() -> tuple[argparse.Namespace, list[str]]:
    """Parse command-line arguments."""
    parser = argparse.ArgumentParser(description="npm static binary")
    return parser.parse_known_args()


def main_npm() -> int:
    """Main entry point for the template_python_cmd package."""
    return run(Npm())


def main_node() -> int:
    return run(Node())


def main_npx() -> int:
    return run(Npx())


def main_npm_tool() -> int:
    _, args = parse_args()
    try:
        proc = NpmTool(args[0]).run(args[1:])
        return proc.wait()
    except KeyboardInterrupt:
        return 1



import platform
from dataclasses import dataclass
from pathlib import Path

from download import download
from filelock import FileLock

from static_npm.paths import CACHE_DIR

LATEST = "22.12.0"


@dataclass
class Binaries:
    npm: Path
    node: Path
    npx: Path


Path(CACHE_DIR).mkdir(parents=True, exist_ok=True)

_LOCK = FileLock((str(CACHE_DIR / "static-npm.lock")))


BASE_URL = "https://github.com/zackees/static-npm/raw/refs/heads/main/npm"

BINARIES_22_12_0 = {
    "darwin": {
        "arm64": f"{BASE_URL}/node-v22.12.0-darwin-arm64.tar.gz",
        "x64": f"{BASE_URL}/node-v22.12.0-darwin-x64.tar.gz",
    },
    "linux": {
        "arm64": f"{BASE_URL}/node-v22.12.0-linux-arm64.tar.xz",
        "armv7l": f"{BASE_URL}/node-v22.12.0-linux-armv7l.tar.xz",
        "x64": f"{BASE_URL}/node-v22.12.0-linux-x64.tar.xz",
    },
    "windows": {
        "x64": f"{BASE_URL}/node-v22.12.0-win-x64.zip",
    },
}

VERSION_BINARIES = {
    "22.12.0": BINARIES_22_12_0,
}


def get_bin_url(version: str, platform: str, arch: str) -> str:
    return VERSION_BINARIES[version][platform][arch]


def get_default(version: str) -> str:
    """Gets the binary URL for the current platform and architecture.

    Args:
        version: The node version to get (e.g. "22.12.0")

    Returns:
        The URL for the appropriate binary

    Raises:
        KeyError: If the platform/architecture combination is not supported
    """
    system = platform.system().lower()
    machine = platform.machine().lower()

    # Map system names to our platform keys
    if system == "darwin":
        platform_key = "darwin"
    elif system == "linux":
        platform_key = "linux"
    elif system == "windows":
        platform_key = "windows"
    else:
        raise KeyError(f"Unsupported platform: {system}")

    # Map CPU architectures to our arch keys
    if machine in ("x86_64", "amd64"):
        arch_key = "x64"
    elif machine == "arm64":
        arch_key = "arm64"
    elif machine == "armv7l":
        arch_key = "armv7l"
    else:
        raise KeyError(f"Unsupported architecture: {machine}")

    return get_bin_url(version, platform_key, arch_key)


def decompress(src: Path) -> Path:
    dst: Path = src.with_suffix("")
    if dst.exists():
        return dst
    src_name = src.name
    if src_name.endswith(".zip"):
        import zipfile

        with zipfile.ZipFile(src, "r") as zip_ref:
            zip_ref.extractall(dst)
    # elif src.suffix == ".tar.gz":
    elif src_name.endswith(".tar.gz"):
        import tarfile

        with tarfile.open(src, "r:gz") as tar_ref:
            tar_ref.extractall(dst)
    # elif src.suffix == ".tar.xz":
    elif src_name.endswith(".tar.xz"):
        import tarfile

        with tarfile.open(src, "r:xz") as tar_ref:
            tar_ref.extractall(dst)
    else:
        raise ValueError(f"Unsupported file type: {src.suffix}")
    return dst


def get_executable(src_dir: Path, name: str) -> Path:
    for path in src_dir.iterdir():
        if path.name.startswith(name):
            return path

    raise FileNotFoundError(f"{name} executable not found")


def ensure_npm_exists(version: str = "22.12.0") -> Binaries:
    with _LOCK:
        # print("Downloading Windows binary...")
        src = get_default(version)
        name = Path(src).name
        dst = CACHE_DIR / name
        # print(f"dst: {dst}")
        if not dst.exists():
            download(src, str(dst), replace=True)
        # print("Download complete.")
        folder = decompress(dst)
        # print(f"Decompressed to: {folder}")
        folder = folder.iterdir().__next__()
        # print(f"folder: {folder}")
        assert folder.exists(), "Decompressed folder does not exist"
        bin = folder / "bin"
        if bin.exists():
            folder = bin
        try:
            npm_path = get_executable(folder, "npm")
            node_path = get_executable(folder, "node")
            npx_path = get_executable(folder, "npx")
        except FileNotFoundError:
            # print out all the files in the folder
            print("Files in folder:")
            for path in folder.iterdir():
                print(path)
            raise
        return Binaries(
            npm=npm_path,
            node=node_path,
            npx=npx_path,
        )



from pathlib import Path

from static_npm.ensure_npm_exists import LATEST, Binaries, ensure_npm_exists
from static_npm.running_process import RunningProcess


class Npm:
    def __init__(self, version: str = LATEST):
        self._binaries: Binaries | None = None
        self.version = version

    @property
    def binaries(self) -> Binaries:
        if self._binaries is None:
            self._binaries = ensure_npm_exists(self.version)
        return self._binaries

    @property
    def path(self) -> Path:
        return Path(self.binaries.npm)

    def global_bin_path(self) -> Path:
        return Path(self.run(["root", "-g"]).stdout.strip())

    def global_prefix_path(self) -> Path:
        return Path(self.run(["prefix", "-g"]).stdout.strip())

    def run(self, cmd_list: list[str], echo=True) -> RunningProcess:
        npm_path = self.binaries.npm
        cmd_list = [str(npm_path)] + cmd_list
        proc = RunningProcess(cmd_list, echo=echo)
        return proc



from .node import Node
from .npm import Npm
from .npm_tool import NpmTool
from .npx import Npx
from .paths import CACHE_DIR
from .running_process import RunningProcess

__all__ = ["Npm", "Node", "Npx", "NpmTool", "RunningProcess", "CACHE_DIR"]



"""
Unit test file.
"""

import os
import unittest

COMMAND = "static-npm --version"


class MainTester(unittest.TestCase):
    """Main tester class."""

    def test_cli_tools(self) -> None:
        """Test command line interface (CLI)."""
        rtn = os.system("static-npm --version")
        self.assertEqual(0, rtn)
        rtn = os.system("static-npm --version")
        self.assertEqual(0, rtn)
        rtn = os.system("static-npm --version")
        self.assertEqual(0, rtn)
        rtn = os.system("static-npm-tool live-server --version")
        self.assertEqual(0, rtn)


if __name__ == "__main__":
    unittest.main()



"""
Setup file.
"""

import os
import re

from setuptools import setup

URL = "https://github.com/zackees/static-npm"
KEYWORDS = "node npm static binaries"
HERE = os.path.dirname(os.path.abspath(__file__))



if __name__ == "__main__":
    setup(
        maintainer="Zachary Vorhies",
        keywords=KEYWORDS,
        url=URL,
        package_data={"": ["assets/example.txt"]},
        include_package_data=True)




"""
Unit test file.
"""

import unittest

from static_npm.npm import Npm


class NpmTester(unittest.TestCase):
    """Main tester class."""

    def test_npm_run_version(self) -> None:
        """Test command line interface (CLI)."""
        npm = Npm()
        proc = npm.run(["--version"])
        rtn = proc.wait()
        version = proc.stdout
        print(f"Version: {version}")
        self.assertIn("10.9.0", version)
        self.assertEqual(0, rtn)
        self.assertEqual(npm.binaries.npm.exists(), True)


if __name__ == "__main__":
    unittest.main()
